@@ -14,7 +14,7 @@ Usage::
 """
 
 
-__version__ = '0.1.1'
+__version__ = '0.1.3'
 __author__ = 'hit9'
 __license__ = 'bsd2'
 
@@ -63,6 +63,7 @@ type_mappings = {
     'hexists': bool,
     'hsize': int,
     'hlist': list,
+    'hrlist': list,
     'hkeys': list,
     'hgetall': list,
     'hscan': list,
@@ -78,6 +79,7 @@ type_mappings = {
     'zexists': bool,
     'zsize': int,
     'zlist': list,
+    'zrlist': list,
     'zkeys': list,
     'zscan': list,
     'zrscan': list,
@@ -88,7 +90,7 @@ type_mappings = {
     'zclear': int,
     'zcount': int,
     'zsum': int,
-    'zavg': int,
+    'zavg': float,
     'zremrangebyrank': int,
     'zremrangebyscore': int,
     'multi_zset': int,
@@ -105,7 +107,10 @@ type_mappings = {
     'qpush_back': int,
     'qpop': str,
     'qpop_front': str,
-    'qpop_back': str
+    'qpop_back': str,
+    'qlist': list,
+    'qrlist': list,
+    'info': list
 }
 
 
@@ -122,6 +127,47 @@ class SSDBException(Exception):
     pass
 
 
+class ResponseParser(object):
+
+    def __init__(self, sock):
+        self.sock = sock
+
+    def read_bytes(self, size):
+        return nativestr(self.sock.recv(size))
+
+    def read_unit(self):
+        length = ''
+        while 1:
+            buf = self.read_bytes(1)
+            if buf == '\n':
+                break
+            length += buf
+        if length:  # when length != ''
+            length = int(length)
+            data = self.read_bytes(length)
+            assert self.read_bytes(1) == '\n'
+            return data
+        return None  # when buf == '\n', length == ''
+
+    def parse(self, resps_count):
+        resp_datas = []
+
+        while resps_count > 0:
+            resp_data = []
+            status = self.read_unit()
+            resp_data.append(status)
+
+            # try to read bodys
+            while 1:
+                body = self.read_unit()
+                if body is None:
+                    break
+                resp_data.append(body)
+            resp_datas.append(resp_data)
+            resps_count -= 1
+        return resp_datas
+
+
 class Connection(threading.local):
     """Ssdb connection object, usage::
 
@@ -137,6 +183,7 @@ class Connection(threading.local):
         self.timeout = timeout
         self.batch_mode = False
         self.__commands = []  # commands sent cache
+        self.parser = None
 
     def connect(self):
         self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -144,6 +191,7 @@ class Connection(threading.local):
         self.sock.connect((self.host, self.port))
         self.sock.setblocking(1)  # block mode
         self.sock.settimeout(self.timeout)
+        self.parser = ResponseParser(self.sock)
 
     def compile(self, args):
         pattern = '{0}\n{1}\n'
@@ -157,14 +205,10 @@ class Connection(threading.local):
         self.sock.sendall(rawstr(command))
 
     def recv(self):
-        data = ''
-        resps_count = len(self.__commands)
-        while resps_count > 0:
-            buf = nativestr(self.sock.recv(1024))
-            data += buf
-            resps_count -= buf.count('\n\n')
+        resp_datas = self.parser.parse(len(self.__commands))
+
         try:
-            resp = self.response(data)
+            resp = self.response(resp_datas)
         except Exception:
             raise
         finally:
@@ -184,14 +228,11 @@ class Connection(threading.local):
         self.send()
         return self.recv()
 
-    def response(self, data):
-        raw_resps = data.strip().split('\n\n')
-        bodys = [raw_resp.splitlines()[1::2] for raw_resp in raw_resps]
-
+    def response(self, resp_datas):
         resps = []
-        for index, lst in enumerate(bodys):
+        for index, resp_data in enumerate(resp_datas):
             command = self.__commands[index][0]
-            status, body = lst[0], lst[1:]
+            status, body = resp_data[0], resp_data[1:]
             resps.append(self.make_response(status, body, command))
 
         if self.batch_mode:
@@ -203,10 +244,12 @@ class Connection(threading.local):
             return None
         elif status == status_ok:
             type = type_mappings.get(command, str)
-            if type in (bool, int, str):
+            if type in (int, float, str):
                 return type(body[0])
+            elif type is bool:
+                return bool(int(body[0]))
             elif type is list:
-                return type(body)
+                return list(body)
         else:
             if body:
                 error_message = '{}: {}'.format(status, body[0])
